var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __async = (__this, __arguments, generator) => {
  return new Promise((resolve, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};
import { defineComponent as _defineComponent } from "vue";
import { computed, provide, reactive, watch } from "vue";
import { tableEditableProps } from "./tableEditable.mjs";
import { PagodaUiError, isPagodaUiError } from "../../utils/error.mjs";
import { objectFromArray, loopTree } from "pagoda-shared";
import { TABLE_EDITABLE_INJECTION_KEY } from "./types/index.mjs";
import { useTableStore } from "./composables/use-table-store.mjs";
import { createNamespace } from "../../utils/index.mjs";
const name = createNamespace("table-editable");
const __default__ = {
  name
};
const __vue_sfc__ = /* @__PURE__ */ _defineComponent(__spreadProps(__spreadValues({}, __default__), {
  props: tableEditableProps,
  emits: ["update:table-data"],
  setup(__props, { expose: __expose, emit: __emit }) {
    const props = __props;
    const emit = __emit;
    let triggerTableWatcherByOwnerAction = false;
    const store = useTableStore({
      data: props.tableData,
      getColumns: () => props.editableColumns,
      childrenKey: props.childrenKey,
      rowStatesCreator(row, parent) {
        return __spreadValues({
          parent,
          editing: props.defaultEditing
        }, props.rowStates(row, parent));
      },
      getCellsStatesCreator: () => {
        return objectFromArray(props.editableColumns, (key) => {
          return (row, parent) => {
            const initCellStates = props.cellsStates[key];
            return __spreadValues({
              editing: props.defaultEditing,
              validateMsg: "",
              hovering: false
            }, initCellStates == null ? void 0 : initCellStates(row, parent));
          };
        });
      }
    });
    watch(
      () => props.tableData,
      (newVal, oldVal) => {
        if (!triggerTableWatcherByOwnerAction && newVal !== oldVal) {
          store.resetStore(props.tableData);
        }
        triggerTableWatcherByOwnerAction = false;
      },
      {
        immediate: true
      }
    );
    watch(
      () => props.editableColumns,
      () => {
        store.resetCellsStates(props.tableData);
      },
      {
        deep: true
      }
    );
    function editRows(rows) {
      store.setRowsStates(rows, {
        editing: true
      });
    }
    function saveRows(rows) {
      saveCells(rows, props.editableColumns);
    }
    function saveCells(rows, cells) {
      cancelCells(rows, cells, false);
    }
    function cancelRows(rows, shouldRollbackData = true) {
      cancelCells(rows, props.editableColumns, shouldRollbackData);
    }
    function cancelCells(rows, cells, shouldRollbackData = true) {
      loopTree(rows, props.childrenKey, (row) => {
        const [rowCacheData] = store.getRowsCache([row]);
        cells.forEach((cell) => {
          if (shouldRollbackData && rowCacheData) {
            if (cell in rowCacheData) {
              row[cell] = rowCacheData[cell];
            } else {
              delete row[cell];
            }
          }
          store.setCellStates([row], [cell], {
            editing: false,
            validateMsg: ""
          });
        });
        store.setRowsCache([row]);
        store.setRowsStates([row], {
          editing: false
        });
      });
    }
    function editColumns(rows, cells) {
      store.setCellStates(rows, cells, {
        editing: true
      });
    }
    function delRows(rows, parent) {
      triggerTableWatcherByOwnerAction = true;
      const data = props.childrenKey && parent ? parent[props.childrenKey] : props.tableData;
      const newData = data.filter((row) => !rows.includes(row));
      if (props.childrenKey && parent) {
        parent[props.childrenKey] = newData;
        emit("update:table-data", props.tableData);
      } else {
        emit("update:table-data", newData);
      }
      store.delRowsStore(rows);
    }
    function newRows(rows, parent) {
      const data = props.childrenKey && parent ? parent[props.childrenKey] : props.tableData;
      const lastRow = data[data.length - 1];
      return insertRowsAfterRow(lastRow, rows, parent);
    }
    function insertRowsBeforeRow(row, rows, parent) {
      return insertRows("before", row, rows, parent);
    }
    function insertRowsAfterRow(row, rows, parent) {
      return insertRows("after", row, rows, parent);
    }
    function insertRows(type, row, rows, parent) {
      const data = props.childrenKey && parent ? parent[props.childrenKey] : props.tableData;
      let insertIdx;
      insertIdx = data.findIndex((_row) => _row === row);
      if (insertIdx === -1) {
        insertIdx = 0;
      } else if (type === "after") {
        insertIdx++;
      }
      triggerTableWatcherByOwnerAction = true;
      rows = rows.map((row2) => reactive(row2));
      const newData = [
        ...data.slice(0, insertIdx),
        ...rows,
        ...data.slice(insertIdx, data.length)
      ];
      if (props.childrenKey && parent) {
        parent[props.childrenKey] = newData;
        emit("update:table-data", props.tableData);
      } else {
        emit("update:table-data", newData);
      }
      store.addStore(rows, parent);
      return rows;
    }
    function validateRows(rows, cb, deep) {
      validateCells(rows, props.editableColumns, cb, deep);
    }
    function validate(cb) {
      validateRows(props.tableData, cb, true);
    }
    const validateCells = function(rows, cells, cb, deep) {
      return __async(this, null, function* () {
        const validatePromiseStacks = [];
        let valid;
        loopTree(rows, deep ? props.childrenKey : "", (row) => {
          const cellsStates = store.getCellsStates(row);
          const rowStates = store.getRowStates(row);
          if (!rowStates || !cellsStates) {
            throw new PagodaUiError(
              "Component TableEditable",
              `\u672A\u627E\u5230\u5F53\u524D\u884C\u72B6\u6001\uFF1A${JSON.stringify(row)}`
            );
          }
          cells.forEach((cell) => {
            const validator = props.validators[cell];
            const ownStates = cellsStates[cell];
            if ((ownStates.editing || rowStates.editing) && validator) {
              const validPropPromise = new Promise((resolve, reject) => {
                const next = (errorMsg) => {
                  if (ownStates.editing || rowStates.editing) {
                    ownStates.validateMsg = errorMsg;
                    if (errorMsg) {
                      reject(
                        new PagodaUiError("Component PagodaTableEditable", errorMsg)
                      );
                    } else {
                      resolve();
                    }
                  }
                };
                validator(row, next, ownStates, rowStates, cellsStates, row[cell]);
              });
              validatePromiseStacks.push(validPropPromise);
            }
          });
        });
        try {
          yield Promise.all(validatePromiseStacks);
          valid = true;
        } catch (e) {
          if (!isPagodaUiError(e)) {
            console.error(e);
          }
          valid = false;
        }
        if (typeof cb === "function")
          cb(valid);
      });
    };
    function isEditing() {
      for (let rowIdx = 0; rowIdx < props.tableData.length; rowIdx++) {
        const row = props.tableData[rowIdx];
        const cellsStates = store.getCellsStates(row);
        const rowStates = store.getRowStates(row);
        if (!rowStates || !cellsStates) {
          throw new PagodaUiError(
            "Component TableEditable",
            `\u672A\u627E\u5230\u5F53\u524D\u884C\u72B6\u6001\uFF1A${JSON.stringify(row)}`
          );
        }
        if (rowStates.editing)
          return true;
        for (let columnIdx = 0; columnIdx < props.editableColumns.length; columnIdx++) {
          const cell = props.editableColumns[columnIdx];
          const cellState = cellsStates[cell];
          if (cellState.editing)
            return true;
        }
      }
      return false;
    }
    provide(TABLE_EDITABLE_INJECTION_KEY, {
      store,
      editableColumns: computed(() => props.editableColumns),
      validateMsgTooltip: computed(() => props.validateMsgTooltip),
      tableData: computed(() => props.tableData),
      validateCells
    });
    __expose({
      editRows,
      editColumns,
      delRows,
      newRows,
      insertRowsBeforeRow,
      insertRowsAfterRow,
      saveRows,
      cancelRows,
      saveCells,
      cancelCells,
      validate,
      validateRows,
      validateCells,
      isEditing,
      store
    });
    const __returned__ = { name, props, emit, get triggerTableWatcherByOwnerAction() {
      return triggerTableWatcherByOwnerAction;
    }, set triggerTableWatcherByOwnerAction(v) {
      triggerTableWatcherByOwnerAction = v;
    }, store, editRows, saveRows, saveCells, cancelRows, cancelCells, editColumns, delRows, newRows, insertRowsBeforeRow, insertRowsAfterRow, insertRows, validateRows, validate, validateCells, isEditing };
    Object.defineProperty(__returned__, "__isScriptSetup", { enumerable: false, value: true });
    return __returned__;
  }
}));
import { renderSlot as _renderSlot, openBlock as _openBlock, createElementBlock as _createElementBlock } from "vue";
const _hoisted_1 = { class: "pagoda-table-editable" };
function __vue_render__(_ctx, _cache, $props, $setup, $data, $options) {
  return _openBlock(), _createElementBlock("div", _hoisted_1, [
    _renderSlot(_ctx.$slots, "default")
  ]);
}
__vue_sfc__.render = __vue_render__;
var stdin_default = __vue_sfc__;
export {
  stdin_default as default
};
