import { defineComponent, h, getCurrentInstance } from "vue";
import { GET_EMPTY_ARR } from "pagoda-shared";
function hGutter(gutterWidth) {
  return h("td", {
    class: "el-table__cell el-table__fixed-right-patch",
    width: gutterWidth
  });
}
function hColgroup(columns, gutterWidth) {
  return h("colgroup", {}, [
    ...columns.map(
      (column) => h("col", {
        name: column.id,
        key: column.id,
        width: column.realWidth || column.width
      })
    ),
    gutterWidth && h("col", {
      name: "gutter",
      width: gutterWidth
    })
  ]);
}
function hCell(row, column, cellIndex, cellSlot) {
  const cellValue = (row.data || [])[column.property];
  return cellSlot ? cellSlot({
    row,
    column
  }) : cellIndex === 0 ? row.label : cellValue;
}
function getOffset(offset, column) {
  return offset + (column.realWidth === null || Number.isNaN(column.realWidth) ? Number(column.width) : column.realWidth);
}
const getFixedColumnOffset = (index, fixed, store, gutterWidth) => {
  const { direction, start = 0 } = isFixedColumn(index, fixed, store);
  if (!direction) {
    return;
  }
  const styles = {};
  const isLeft = direction === "left";
  const columns = store.states.columns.value;
  if (isLeft) {
    styles.left = columns.slice(0, index).reduce(getOffset, 0);
  } else {
    styles.right = columns.slice(start + 1).reverse().reduce(getOffset, 0) + gutterWidth;
  }
  return styles;
};
const isFixedColumn = (index, fixed, store, realColumns) => {
  let start = 0;
  let after = index;
  if (realColumns) {
    for (let i = 0; i < index; i++) {
      start += realColumns[i].colSpan;
    }
    after = start + realColumns[index].colSpan - 1;
  } else {
    start = index;
  }
  let fixedLayout;
  const { columns } = store.states;
  switch (fixed) {
    case "left":
      if (after < store.states.fixedLeafColumnsLength.value) {
        fixedLayout = "left";
      }
      break;
    case "right":
      if (start >= columns.value.length - store.states.rightFixedLeafColumnsLength.value) {
        fixedLayout = "right";
      }
      break;
    default:
      if (after < store.states.fixedLeafColumnsLength.value) {
        fixedLayout = "left";
      } else if (start >= columns.value.length - store.states.rightFixedLeafColumnsLength.value) {
        fixedLayout = "right";
      }
  }
  return fixedLayout ? {
    direction: fixedLayout,
    start,
    after
  } : {};
};
const getFixedColumnsClass = (index, fixed, store, realColumns) => {
  const classes = [];
  const { direction, start } = isFixedColumn(index, fixed, store, realColumns);
  if (direction) {
    const isLeft = direction === "left";
    classes.push(`el-table-fixed-column--${direction}`);
    if (isLeft && start === store.states.fixedLeafColumnsLength.value - 1) {
      classes.push("is-last-column");
    } else if (!isLeft && start === store.states.columns.value.length - store.states.rightFixedLeafColumnsLength.value) {
      classes.push("is-first-column");
    }
  }
  return classes;
};
const ensurePosition = (style, key) => {
  if (!style)
    return;
  if (!Number.isNaN(style[key])) {
    style[key] = `${style[key]}px`;
  }
};
var stdin_default = defineComponent({
  props: {
    fixed: {
      type: String,
      default: ""
    },
    table: {
      type: Object,
      required: true
    },
    data: {
      type: Array,
      default: GET_EMPTY_ARR
    }
  },
  setup(props) {
    const instance = getCurrentInstance();
    return () => {
      const columns = props.table.store.states.columns.value;
      const { bodyWidth } = props.table;
      const { gutterWidth } = props.table.layout;
      const { data } = props;
      return h(
        "table",
        {
          class: "el-table__body",
          cellspacing: "0",
          cellpadding: "0",
          border: "0",
          style: {
            width: bodyWidth
          }
        },
        [
          hColgroup(columns, gutterWidth),
          h(
            "tbody",
            data.map((row, rowIndex) => {
              return h("tr", {}, [
                ...columns.map((column, columnIndex) => {
                  const fixedClasses = column.isSubColumn ? [] : getFixedColumnsClass(
                    columnIndex,
                    column.fixed,
                    props.table.store
                  );
                  const classes = [
                    column.id,
                    column.align,
                    column.className,
                    ...fixedClasses
                  ];
                  const { cellClassName } = props.table;
                  if (typeof cellClassName === "string") {
                    classes.push(cellClassName);
                  } else if (typeof cellClassName === "function") {
                    classes.push(
                      cellClassName.call(null, {
                        rowIndex,
                        columnIndex,
                        row,
                        column
                      })
                    );
                  }
                  const fixedStyle = getFixedColumnOffset(
                    columnIndex,
                    column.fixed,
                    props.table.store,
                    gutterWidth
                  );
                  ensurePosition(fixedStyle, "left");
                  ensurePosition(fixedStyle, "right");
                  return h(
                    "td",
                    {
                      key: columnIndex,
                      colspan: column.colSpan,
                      rowspan: column.rowSpan,
                      style: fixedStyle,
                      class: ["el-table__cell", ...classes]
                    },
                    [
                      h(
                        "div",
                        {
                          class: ["cell", column.labelClassName]
                        },
                        [
                          hCell(
                            row,
                            column,
                            columnIndex,
                            instance.parent.slots[column.property]
                          )
                        ]
                      )
                    ]
                  );
                }),
                gutterWidth && hGutter(gutterWidth)
              ]);
            })
          )
        ]
      );
    };
  }
});
export {
  stdin_default as default
};
