var __async = (__this, __arguments, generator) => {
  return new Promise((resolve, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};
import { ref, reactive } from "vue";
function useTableRequest({
  initialPagination,
  onRequest
}) {
  const tableData = ref([]);
  const pagination = reactive(
    Object.assign(
      {},
      {
        currentPage: 1,
        total: 0,
        pageSize: 10
      },
      initialPagination
    )
  );
  let tableSortable;
  function triggerRequest() {
    return __async(this, null, function* () {
      const { currentPage, pageSize } = pagination;
      const p = onRequest(currentPage, pageSize, tableSortable);
      const res = yield Promise.resolve(p);
      if (res) {
        const { data, total } = res;
        tableData.value = data;
        pagination.total = total;
      }
    });
  }
  function refresh() {
    return __async(this, null, function* () {
      pagination.currentPage = 1;
      yield refreshCurrentPage();
    });
  }
  function refreshCurrentPage() {
    return __async(this, null, function* () {
      yield triggerRequest();
    });
  }
  function setTableSortable(sortable) {
    tableSortable = sortable;
    triggerRequest();
  }
  function setPaginationSize(pageSize) {
    pagination.pageSize = pageSize;
    refresh();
  }
  function setPaginationCurrentPage(currentPage) {
    pagination.currentPage = currentPage;
    triggerRequest();
  }
  return {
    // states
    tableData,
    pagination,
    // functions
    refresh,
    refreshCurrentPage,
    setTableSortable,
    setPaginationSize,
    setPaginationCurrentPage
  };
}
export {
  useTableRequest
};
