import { ref } from "vue";
import { toggleRowStatus } from "../utils.mjs";
import { debounce } from "lodash-unified";
function useSelection({
  tableData,
  selectable,
  onSelectionChange
}) {
  const selection = ref([]);
  const isAllSelected = ref(false);
  const getSelectionRows = () => {
    return (selection.value || []).slice();
  };
  const isSelected = (row) => {
    return selection.value.includes(row);
  };
  const toggleRowSelection = (row, selected) => {
    const changed = toggleRowStatus(selection.value, row, selected);
    if (changed) {
      const newSelection = (selection.value || []).slice();
      onSelectionChange == null ? void 0 : onSelectionChange(newSelection);
    }
  };
  const updateAllSelected = () => {
    var _a;
    if (((_a = tableData.value) == null ? void 0 : _a.length) === 0) {
      isAllSelected.value = false;
      return;
    }
    const isSelected2 = function(row) {
      return selection.value.includes(row);
    };
    let isAllSelected_ = true;
    let selectedCount = 0;
    for (let i = 0, j = (tableData.value || []).length; i < j; i++) {
      const rowIndex = i;
      const item = tableData.value[i];
      const isRowSelectable = selectable.value && selectable.value.call(null, item, rowIndex);
      if (!isSelected2(item)) {
        if (!selectable.value || isRowSelectable) {
          isAllSelected_ = false;
          break;
        }
      } else {
        selectedCount++;
      }
    }
    if (selectedCount === 0)
      isAllSelected_ = false;
    isAllSelected.value = isAllSelected_;
  };
  const clearSelection = () => {
    isAllSelected.value = false;
    const oldSelection = selection.value;
    if (oldSelection.length) {
      selection.value = [];
      onSelectionChange == null ? void 0 : onSelectionChange([]);
    }
  };
  const toggleAllSelection = debounce(() => {
    const value = !isAllSelected.value;
    isAllSelected.value = value;
    let selectionChanged = false;
    tableData.value.forEach((row, index) => {
      if (selectable.value) {
        if (selectable.value.call(null, row, index) && toggleRowStatus(selection.value, row, value)) {
          selectionChanged = true;
        }
      } else {
        if (toggleRowStatus(selection.value, row, value)) {
          selectionChanged = true;
        }
      }
    });
    if (selectionChanged) {
      onSelectionChange == null ? void 0 : onSelectionChange(selection.value ? selection.value.slice() : []);
    }
  }, 10);
  return {
    selection,
    isAllSelected,
    getSelectionRows,
    isSelected,
    toggleRowSelection,
    clearSelection,
    updateAllSelected,
    toggleAllSelection
  };
}
export {
  useSelection
};
