export type UseTableStoreStateCreator = (row: RowData, parent?: RowData) => UseTableStoreRowConfig['states'];
export type UseTableStoreData = RowData[];
export type RowData = Record<string, any>;
export type CellsStatesCreator = Record<string, UseTableStoreStateCreator>;
export interface UseTableStoreParams {
    data?: UseTableStoreData;
    getColumns?: () => string[];
    rowStatesCreator?: UseTableStoreStateCreator;
    getCellsStatesCreator?: () => CellsStatesCreator;
    childrenKey?: string;
}
export interface UseTableStoreRowConfig<T = Record<string, any>, U = Record<string, any>> {
    states: T;
    cellsStates: Record<string, U>;
}
export declare function useTableStore(options: UseTableStoreParams): {
    states: WeakMap<RowData, UseTableStoreRowConfig<Record<string, any>, Record<string, any>>>;
    tableCacheData: WeakMap<RowData, RowData>;
} & {
    addStore: (data: UseTableStoreData, parent?: RowData) => void;
    resetStore: (data: UseTableStoreData) => void;
    setRowsStates: (rows: RowData[], states: UseTableStoreRowConfig["states"]) => void;
    setCellStates: (rows: RowData[], cells: string[], states: UseTableStoreRowConfig["states"]) => void;
    resetCellsStates: (data: UseTableStoreData) => void;
    setRowsCache: (rows: RowData[]) => void;
    getRowsCache: (rows: RowData[]) => (RowData | undefined)[];
    getCellStates: (row: RowData, cell: string) => Record<string, any> | undefined;
    getRowStates: (row: RowData) => Record<string, any> | undefined;
    getCellsStates: (row: RowData) => Record<string, Record<string, any>> | undefined;
    delRowsStates: (rows: RowData[]) => void;
    deleteRowsCache: (rows: RowData[]) => void;
    delRowsStore: (rows: RowData[]) => void;
};
